from .guild import Guild, Channel, Emoji
from .role import Role
from .member import Member, Presence

class AllGuild(Guild):
  def __init__(self,instance,guild,cdn,token,url):
    super().__init__(guild,cdn,instance)
    self.instance = instance
    self.channels = []
    for channel in guild['channels']:
      self.channels.append(Channel(channel,instance))
    self.emojis = []
    for emoji in guild['emojis']:
      self.emojis.append(Emoji(emoji,instance))
    self.roles = []
    for role in guild['roles']:
      self.roles.append(Role(role))
    self.members = []
    for member in guild['members']:
      self.members.append(Member(instance,member,cdn,token,url,self.id))
    self.large = guild['large']
    self.member_count = guild['member_count']
    self.presences = []
    for presence in guild['presences']:
      self.presences.append(Presence(instance,presence,cdn,token,url,self.id))
    self.unavailable = guild.get('unavailable') #nonexistent on guild join mid-session

  async def toDict(self):
    return {"channels":[channel.toDict() for channel in self.channels], "emojis":[emoji.toDict() for emoji in self.emojis], "roles":[role.toDict() for role in self.roles], "members":[member.toDict() for member in self.members], "large":self.large, "member_count":self.member_count, "presences":[presence.toDict() for presence in self.presences],"unavailable":self.unavailable}


import discord
from .role import Role

class Permissions(discord.Permissions):
  def __init__(self,role=None):
    if type(role) == Role:
      role.permissions.update()
      super().__init__(role.permissions.value) #yeah no lets just skid discord.py :trol:
    elif type(role) == int:
      super().__init__(role)
    elif type(role) == str:
      super().__init__(int(role))
    elif not role: #None
      super().__init__(0) #creation of a fake role


import requests
import fake_useragent
import json
from .message import Message

ua = fake_useragent.UserAgent(browsers=['chrome', "firefox", "opera", "safari", "edge", "internet explorer"])

class Fetch:
  def __init__(self,token,url,cdn,instance):
    self.token = token
    self.base_url = url
    self.cdn = cdn
    self.instance = instance
    self.agent = ua.random
  
  async def getMessage(self,channel,id):
    headers = {
      'Authorization': f'Bot {self.token}',
      'Content-Type': 'application/json',
      'User-Agent': self.agent
    }
    data = {"limit":100}
    e = requests.get(url=f"{self.base_url}channels/{channel}/messages/",headers=headers,json=data)
    for message in e.json():
      if id == message['id']:
        return Message(message,self.token,self.agent,self.base_url,self.cdn,self.instance)

  async def getMessages(self,channel,limit):
    headers = {
      'Authorization': f'Bot {self.token}',
      'Content-Type': 'application/json',
      'User-Agent': self.agent
    }
    data = {"limit":limit}
    e = requests.get(url=f"{self.base_url}channels/{channel}/messages/",headers=headers,json=data)
    messages = []
    i = 0
    for message in e.json():
      if i < limit:
        messages.append(Message(message,self.token,self.agent,self.base_url,self.cdn,self.instance))
      i += 1
    return messages


import requests
from PIL.Image import open as openImage
from io import BytesIO

def getDimensions(image_url):
	response = requests.get(image_url)
	if response.status_code == 200:
					img = openImage(BytesIO(response.content))
					width, height = img.size
					return width, height
	else:
					raise Exception(f"Failed to retrieve image from URL. Status code: {response.status_code}")

class Field:
	def __init__(self,name,value,inline=False):
		self.name = name
		self.value = value
		self.inline = inline

class Footer:
	def __init__(self,text,icon_url=None):
		self.text = text
		self.icon_url = icon_url

class EmbedAuthor:
	def __init__(self,name,url=None,icon_url=None):
		self.name = name
		self.url = url
		self.icon_url = icon_url

class Thumbnail:
	def __init__(self,url):
		self.url = url
		self.width,self.height = getDimensions(url)

class Image:
	def __init__(self,url):
		self.url = url
		self.width,self.height = getDimensions(url)

class Embed:
	def __init__(self,title,description,color:int=0,url=None,timestamp=None):
		self.title = title
		self.description = description
		self.color = color
		self.fields = []
		self.url = url
		self.timestamp = timestamp
		self.author = None
		self.footer = None
		self.thumbnail = None
		self.image = None

	async def addAuthor(self,name,url=None,icon_url=None):
		self.author = EmbedAuthor(name,url,icon_url)

	async def addFooter(self,text,icon_url=None):
		self.footer = Footer(text,icon_url)

	async def addField(self,name,value,inline=False):
		self.fields.append(Field(name,value,inline))

	async def addThumbnail(self,url):
		self.thumbnail = Thumbnail(url)

	async def addImage(self,url):
		self.image = Image(url)

#example embed
[{
				"title":
				"Holy crapple!",
				"description":
				"Test works!",
				"color":
				16711680,  # Color in decimal (e.g., Red)
				"fields": [{
								"name": "average extra uneccessary info",
								"value": "ignore"
				}, {
								"name": "but wait",
								"value": "theres more?"
				}]
}]


from .member import User
from .guild import Guild
from .embed import Embed
from .file import File
import requests
import json
import asyncio
from requests_toolbelt import MultipartEncoder
import filetype
import random
import string

async def prepareEmbed(embed):
  prep_embed = {"title":embed.title,"description":embed.description,"color":embed.color,"url":embed.url,"timestamp":embed.timestamp,"fields":[]}
  if embed.image:
    prep_embed["image"] = {"url":embed.image.url,"width":embed.image.width,"height":embed.image.height}
  if embed.thumbnail:
    prep_embed["thumbnail"] = {"url":embed.thumbnail.url,"width":embed.thumbnail.width,"height":embed.thumbnail.height}
  if embed.footer:
    prep_embed["footer"] = {"text":embed.footer.text,"icon_url":embed.footer.icon_url}
  if embed.author:
    prep_embed["author"] = {"name":embed.author.name,"url":embed.author.url,"icon_url":embed.author.icon_url}
  for field in embed.fields:
    prep_embed['fields'].append({"name":field.name,"value":field.value,"inline":field.inline})
  embed = [prep_embed]
  return embed

class Message:
    def __init__(self,data,token,agent,base_url,cdn,instance,reply:bool=False,reply_content=None,reply_author:User=None):
        if reply:
          self.is_reply = True
          self.original_reply = reply_content
          self.original_author = reply_author
        else:
          self.is_reply = False
        self.instance = instance
        self.token = token
        self.agent = agent
        self.base_url = base_url
        self.content = data['content']
        self.channel_id = data['channel_id']
        for guild in instance.allGuilds:
          if data.get("guild_id") == guild.id:
            for channel in guild.channels:
              if data['channel_id'] == channel.id:
                self.channel = channel
        self.guild_id = data.get("guild_id")
        self.dm = False
        if self.guild_id == None:
          self.dm = True
        self.id = data['id']
        self.mention_roles = []
        for role in data['mention_roles']:
          for guild in self.instance.allGuilds:
            if self.guild_id == guild.id:
              for r in guild.roles:
                if r.id == role:
                  self.mention_roles.append(r)
        self.mentions = []
        for mention in data['mentions']:
          self.mentions.append(User(instance,mention,cdn,self.token,self.base_url))
        self.attachments = data['attachments']
        self.embeds = data['embeds']
        self.edited_timestamp = data['edited_timestamp']
        self.pinned = data['pinned']
        self.mention_everyone = data['mention_everyone']
        self.tts = data['tts']
        self.timestamp = data['timestamp']
        self.webhook_id = data['webhook_id']
        self.author = User(instance,data['author'],cdn,self.token,self.base_url,self.guild_id)
        self.cdn = cdn

    async def typing(self):
      headers = {
        'Authorization': f'Bot {self.token}',
        'Content-Type': 'application/json',
        'User-Agent': self.agent
      }
      requests.post(f"{self.base_url}/channels/{self.channel_id}/typing",headers=headers) #returns 204

    async def delete(self):
      headers = {
        'Authorization': f'Bot {self.token}',
        'Content-Type': 'application/json',
        'User-Agent': self.agent
      }
      e = requests.delete(url=f"{self.base_url}channels/{self.channel_id}/messages/{self.id}",headers=headers)
      return e


    async def edit(self, content):
      headers = {
        'Authorization': f'Bot {self.token}',
        'Content-Type': 'application/json',
        'User-Agent': self.agent
      }
      if self.is_reply:
        data = {"content":f"> {self.original_reply} \n<@{self.original_author.id}> {content}"}
      else:
        data = {"content":content}
      r = requests.patch(f"{self.base_url}channels/{self.channel_id}/messages/{self.id}",headers=headers,data=json.dumps(data))
      if self.is_reply:
        return Message(r.json(),self.token,self.agent,self.base_url,self.cdn,self.instance,reply=True,reply_content=content,reply_author=self.original_author)
      else:
        return Message(r.json(),self.token,self.agent,self.base_url,self.cdn,self.instance,reply=False)

    async def send(self, content=None, embed:Embed=None,file:File=None):
      if content == embed == file == None:
        raise Exception("Please inlcude either a message content, embed, or file.") #will be caught by the exception handler in main.py and formatted correctly
      if file: #purposefully prioritizes files over embeds
        if type(file) != File:
          raise Exception("Please pass a \"File\" object to the \"file\" parameter.")
        if embed:
          print("\033[91mIgnoring exception: Cannot send embed and file at the same time. Will send file only.\033[0m") #code does not need to stop, so no exception raised
        headers = file.headers
        headers['Authorization'] = f'Bot {self.token}'
        headers['User-Agent'] = self.agent
        data = {'content': content,'tts':False,'file':file.file_json}
        fields = file.fields
        fields['payload_json'] = (None,json.dumps(data))
        data = MultipartEncoder(fields=fields,boundary='----WebKitFormBoundary'+''.join(random.sample(string.ascii_letters+string.digits,16)))
        headers['Content-Type'] = data.content_type
      else:
        if embed:
          embed = await prepareEmbed(embed)
        headers = {
          'Authorization': f'Bot {self.token}',
          'Content-Type': 'application/json',
          'User-Agent': self.agent,
          }
        data = json.dumps({'content': content,
                           'embeds': embed,
                              'tts': False})
      e = requests.post(url=f"{self.base_url}channels/{self.channel_id}/messages",headers=headers,data=data)
      return Message(e.json(),self.token,self.agent,self.base_url,self.cdn,self.instance,reply=False)

    async def reply(self, content=None,embed:Embed=None,file=None):
      if content == embed == file == None:
        raise Exception("Please inlcude either a message content, embed, or file.")
      if file:
        headers = file.headers
        headers['Authorization'] = f'Bot {self.token}'
        headers['User-Agent'] = self.agent
        data = {'content': f'> {self.content} \n<@{self.author.id}> {content}','tts':False,'file':file.file_json}
        fields = file.fields
        fields['payload_json'] = (None,json.dumps(data))
        data = MultipartEncoder(fields=fields,boundary='----WebKitFormBoundary'+''.join(random.sample(string.ascii_letters+string.digits,16)))
        headers['Content-Type'] = data.content_type
      else:
        if embed:
          embed = await prepareEmbed(embed)
        headers = {
          'Authorization': f'Bot {self.token}',
          'Content-Type': 'application/json',
          'User-Agent': self.agent
          }
        data = json.dumps({'content': f"> {self.content} \n<@{self.author.id}> {content}",
                              'embeds': embed,
                              'tts': False})
      e = requests.post(url=f"{self.base_url}channels/{self.channel_id}/messages",headers=headers,data=data)
      return Message(e.json(),self.token,self.agent,self.base_url,self.cdn,self.instance,reply=True,reply_content=self.content,reply_author=self.author)

    async def getUser(self,id):
      headers = {
        'Authorization': f'Bot {self.token}',
        'Content-Type': 'application/json',
        'User-Agent': self.agent
        }
      e = requests.get(url=f"{self.base_url}users/{id}/",headers=headers)
      if self.guild_id != None:
        for guild in self.instance.allGuilds:
          if self.guild_id == guild.id:
            for member in guild.members:
              if member.id == id:
                return member
      try:
        return User(self.instance,e.json(),self.cdn,self.token,self.base_url,self.guild_id)
      except Exception as f:
        print(f"could not create author class: {f}")
        if e.json()['code'] == 10001:
          pass

    async def getGuildUser(self,user_id):
      user = await self.getUser(user_id)
      for guild in self.instance.allGuilds:
        if guild.id == user.guild_id:
          for member in guild.members:
            if user_id == member.id:
              return member

    async def getGuild(self,guild):
      headers = {
        'Authorization': f'Bot {self.token}',
        'Content-Type': 'application/json',
        'User-Agent': self.agent
        }
      e = requests.get(url=f"{self.base_url}guilds/{guild}",headers=headers)
      #print(e.json())
      #try:
      guild = Guild(e.json(),self.cdn)
      print(guild.id)
        #return channel
      #except Exception:

    async def getGuildChannels(self):
      channels = []
      for guild in self.instance.allGuilds:
        if guild.id == self.guild_id:
          for channel in guild.channels:
            channels.append(channel)
      return channels

    async def getMessage(self,id):
      headers = {
        'Authorization': f'Bot {self.token}',
        'Content-Type': 'application/json',
        'User-Agent': self.agent
      }
      data = {"limit":100} #limit doesnt work but hell who cares
      e = requests.get(url=f"{self.base_url}channels/{self.channel_id}/messages/",headers=headers,json=data)
      for message in e.json():
        if id == message['id']:
          return Message(message,self.token,self.agent,self.base_url,self.cdn,self.instance)

    async def getMessages(self,channel,limit):
      try:
        limit = int(limit)
      except Exception:
        raise Exception("Limit must be an integer.")
      headers = {
        'Authorization': f'Bot {self.token}',
        'Content-Type': 'application/json',
        'User-Agent': self.agent
      }
      data = {"limit":limit}
      e = requests.get(url=f"{self.base_url}channels/{channel}/messages/",headers=headers,json=data)
      messages = []
      i = 0
      for message in e.json():
        if i < limit:
          messages.append(Message(message,self.token,self.agent,self.base_url,self.cdn,self.instance))
        i += 1
      return messages

    async def deleteMessage(self,id):
      headers = {
        'Authorization': f'Bot {self.token}',
        'Content-Type': 'application/json',
        'User-Agent': self.agent
      }
      e = requests.delete(url=f"{self.base_url}channels/{self.channel_id}/messages/{id}",headers=headers)
      return e

    async def bulkDelete(self,messages):
      if len(messages) > 100:
        raise Exception("Please provide less than 100 messages!") #1. no spam api!!! 2. i believe this is the limit for bulk delete back then
      ids = []
      for message in messages:
        ids.append(message.id)
      headers = {
        'Authorization': f'Bot {self.token}',
        'Content-Type': 'application/json',
        'User-Agent': self.agent
      }
      for id in ids:
       requests.delete(url=f"{self.base_url}channels/{self.channel_id}/messages/{id}",headers=headers) #bulk delete endpoint doesnt exist (absolute stupid)


from .role import Role

import fake_useragent
import requests
import json

ua = fake_useragent.UserAgent(browsers=['chrome', "firefox", "opera", "safari", "edge", "internet explorer"])
agent = ua.random

class Avatar:
  def __init__(self,id,avatar,cdn):
    self.id = id
    self.avatar = avatar
    self.url = f"{cdn}avatars/{self.id}/{self.avatar}.png"

class User:
  def __init__(self,instance,data,cdn,token,url,guild_id=None):
    self.instance = instance
    self.guild_id = guild_id
    self.base_url = url
    self.token = token
    self.id = data['id']
    self.username = data['username']
    self.discriminator = data['discriminator']
    self.mention = f"<@{data['id']}>"
    self.avatar = Avatar(self.id,data['avatar'],cdn)
    self.bot = data['bot']

  async def toDict(self):
    return {"id":self.id,"username":self.username,"discriminator":self.discriminator,"mention":self.mention,"avatar":self.avatar.url,"bot":self.bot}

  async def setRoles(self,roles:list):
    headers = {
      'Authorization': f'Bot {self.token}',
      'Content-Type': 'application/json',
      'User-Agent': agent,
    }
    data = json.dumps({'roles': roles})
    e = requests.patch(url=f"{self.base_url}guilds/{self.guild_id}/members/{self.id}",headers=headers,data=data)
    return e

  async def removeRoles(self,roles:list):
    temp = []
    for guild in self.instance.allGuilds:
      if guild.id == self.guild_id:
        for member in guild.members:
          if self.id == member.id:
            temp = member.roles
    idx = 0
    for role in roles:
      if type(roles[0]) == Role:
        if role.id in temp:
          temp.pop(idx)
      else:
        if role in temp:
          temp.pop(idx)
      idx += 1
    roles = temp
    headers = {
      'Authorization': f'Bot {self.token}',
      'Content-Type': 'application/json',
      'User-Agent': agent,
    }
    data = json.dumps({'roles': roles})
    e = requests.patch(url=f"{self.base_url}guilds/{self.guild_id}/members/{self.id}",headers=headers,data=data)
    return e

  async def addRoles(self,roles:list):
    temp = []
    for guild in self.instance.allGuilds:
      if guild.id == self.guild_id:
        for member in guild.members:
          if self.id == member.id:
            temp = member.roles
    if type(roles[0]) == Role:
      for role in roles:
        temp.append(role.id)
      roles = temp
    headers = {
      'Authorization': f'Bot {self.token}',
      'Content-Type': 'application/json',
      'User-Agent': agent,
    }
    data = json.dumps({'roles': roles})
    print(self.guild_id)
    e = requests.patch(url=f"{self.base_url}guilds/{self.guild_id}/members/{self.id}",headers=headers,data=data)
    print(e.json())
    return e

  async def kick(self):
    headers = {
      'Authorization': f'Bot {self.token}',
      'Content-Type': 'application/json',
      'User-Agent': agent
      }
    e = requests.delete(url=f"{self.base_url}guilds/{self.guild_id}/members/{self.id}", headers=headers)
    return e

  async def nick(self,nick):
    headers = {
        'Authorization': f'Bot {self.token}',
        'Content-Type': 'application/json',
        'User-Agent': agent,
        }
    if nick == "None":
        data = json.dumps({'nick': ""})
    else:
        data = json.dumps({'nick': nick})
    e = requests.patch(url=f"{self.base_url}guilds/{self.guild_id}/members/{self.id}",headers=headers,data=data)
    return e

class Member:
  def __init__(self,instance,data,cdn,token,url,guild):
    self.instance = instance
    self.guild_id = guild
    self.id = data['id']
    self.mention = f"<@{data['id']}>"
    self.nick = data['nick']
    self.roles = data['roles']
    self.joined_at = data['joined_at']
    self.deaf = data['deaf']
    self.mute = data['mute']
    self.user = User(instance,data['user'],cdn,token,url,guild)

  async def setRoles(self,roles:list):
    headers = {
      'Authorization': f'Bot {self.instance.token}',
      'Content-Type': 'application/json',
      'User-Agent': agent,
    }
    data = json.dumps({'roles': roles})
    e = requests.patch(url=f"{self.instance.base_url}guilds/{self.guild_id}/members/{self.id}",headers=headers,data=data)
    return e

  async def removeRoles(self,roles:list):
    temp = []
    for guild in self.instance.allGuilds:
      if guild.id == self.guild_id:
        for member in guild.members:
          if self.id == member.id:
            temp = member.roles
    idx = 0
    for role in roles:
      if type(roles[0]) == Role:
        if role.id in temp:
          temp.pop(idx)
      else:
        if role in temp:
          temp.pop(idx)
      idx += 1
    roles = temp
    headers = {
      'Authorization': f'Bot {self.instance.token}',
      'Content-Type': 'application/json',
      'User-Agent': agent,
    }
    data = json.dumps({'roles': roles})
    e = requests.patch(url=f"{self.instance.base_url}guilds/{self.guild_id}/members/{self.id}",headers=headers,data=data)
    return e

  async def addRoles(self,roles:list):
    temp = []
    for guild in self.instance.allGuilds:
      if guild.id == self.guild_id:
        for member in guild.members:
          if self.id == member.id:
            temp = member.roles
    if type(roles[0]) == Role:
      for role in roles:
        temp.append(role.id)
      roles = temp
    headers = {
      'Authorization': f'Bot {self.instance.token}',
      'Content-Type': 'application/json',
      'User-Agent': agent,
    }
    data = json.dumps({'roles': roles})
    print(self.user.guild_id)
    e = requests.patch(url=f"{self.instance.base_url}guilds/{self.guild_id}/members/{self.id}",headers=headers,data=data)
    print(e.json())
    return e

  async def kick(self):
    headers = {
      'Authorization': f'Bot {self.instance.token}',
      'Content-Type': 'application/json',
      'User-Agent': agent,
      }
    e = requests.delete(url=f"{self.user.base_url}guilds/{self.guild_id}/members/{self.id}", headers=headers)
    return e

  async def nick(self,nick):
    headers = {
        'Authorization': f'Bot {self.user.token}',
        'Content-Type': 'application/json',
      'User-Agent': agent,
        }
    if nick == "None":
        data = json.dumps({'nick': ""})
    else:
        data = json.dumps({'nick': nick})
    e = requests.patch(url=f"{self.user.base_url}/guilds/{self.guild_id}/members/{self.id}",headers=headers,data=data)
    return e

class Presence:
  def __init__(self,instance,data,cdn,token,url,guild):
    self.instance = instance
    self.guild_id = guild
    self.game = data['game']
    self.status = data['status']
    self.user = User(instance,data['user'],cdn,token,url,guild)


#quick functions for doing some actions such as getting guild user or role objects and/or comparing them

from .permissions import Permissions
from .allguild import AllGuild
from .member import Member
from .role import FakeRole

async def checkPerms(ctx,allGuilds,perm):
  for guild in allGuilds:
    if guild.id == ctx.guild_id:
      selected = guild
  gmember = await ctx.getGuildUser(ctx.author.id)
  for mrole in gmember.roles:
    for role in selected.roles:
      if role.id == mrole:
        temp = Permissions(role)
        try:
          if getattr(temp,"administrator"):
            return True
          return getattr(temp, perm)
        except AttributeError:
          raise AttributeError(f"{perm} is not a valid permission!")
  return False

async def getRole(role_id,guild:AllGuild):
  for role in guild.roles:
    if role.id == role_id:
      return role

async def checkPositions(user1:Member,user2:Member,allGuilds):
  for guild in allGuilds:
    if guild.id == user1.guild:
      selected = guild
  if len(user1.roles) > 0:
    highest1 = await getRole(user1.roles[0],selected)
    for role in user1.roles:
      fetched = await getRole(role,selected)
      if fetched.position < highest1.position:
        highest1 = fetched
  else:
    highest1 = FakeRole(999999) #who the hell in their right mind is going to make 999999 roles just to fuck with this

  if len(user2.roles) > 0:
    highest2 = await getRole(user2.roles[0],selected)
    for role in user2.roles:
      fetched = await getRole(role,selected)
      if fetched.position < highest1.position:
        highest1 = fetched
  else:
    highest2 = FakeRole(999999)
      
  return highest1.position < highest2.position

async def fullPermsCheck(ctx,instance,perm,target_id=None):
    if ctx.guild_id:
      for guild in instance.allGuilds:
        if guild.id == ctx.guild_id:
          if guild.owner_id == ctx.author.id:
            return True
    perms = await checkPerms(ctx,instance.allGuilds,perm)
    if perms:
      if target_id is not None:
          member = await ctx.getGuildUser(target_id)
          author = await ctx.getGuildUser(ctx.author.id)
          pos = await checkPositions(author,member,instance.allGuilds)
          if pos:
            return True
      else:
        return True #perm checking does not have a target user, no comparison necessary
    return False


class Role:
  def __init__(self,data):
    self.id = data['id']
    self.name = data['name']
    self.color = data['color']
    from .permissions import Permissions #circular import begone
    self.permissions = Permissions(data['permissions'])
    self.mentionable = data['mentionable']
    self.hoist = data['hoist']
    self.position = data['position']

class FakeRole: #this is really used to provide a placeholder for top role comparison in funcs.py
  def __init__(self,position):
    self.position = position


class Events:
  def __init__(self,instance):
    self.instance = instance

  async def on_ready(self, bot):
    pass

  async def on_message_create(self, message):
    pass

  async def on_message_delete(self, message_id, channel_id, guild_id):
    pass

  async def on_message_update(self, message, author):
    pass

  async def on_presence_update(self, before, presence):
    pass

  async def on_guild_member_add(self, guild, member):
    pass

  async def on_guild_member_remove(self, guild, member): #unlike other events, a member object actually gets passed through! discord is not being obscure with info for once!?!?!?
    pass

  async def on_guild_member_update(self, before, after):
    pass

  async def on_guild_create(self, guild):
    pass

  async def on_guild_delete(self, guild):
    pass

  async def on_guild_update(self, before, after):
    pass

  async def on_guild_role_create(self, guild_id, role):
    pass

  async def on_guild_role_delete(self, guild_id, role):
    pass

  async def on_guild_role_update(self, guild_id, before, after):
    pass

  async def on_guild_emojis_update(self, before, after): #emoji object contains guild ID
    pass

  async def on_typing_start(self, channel, user, when):
    pass

  async def on_channel_create(self, channel):
    pass

  async def on_channel_delete(self, channel):
    pass

  async def on_channel_update(self, before, after):
    pass


#welcome to stupidity central
import json
import requests
from websockets.exceptions import ConnectionClosed
from websockets.sync.client import connect
import fake_useragent
import traceback
import inspect
import asyncio
import base64

from .message import Message
from .role import Role
from .member import User, Member, Presence
from .guild import Guild, Channel, Emoji
from .events import Events
from .allguild import AllGuild
from .fetch import Fetch

ua = fake_useragent.UserAgent(browsers=['chrome', "firefox", "opera", "safari", "edge", "internet explorer"])
agent = ua.random


def splitArgs(function, command, name, instance, mentions,role_mentions):
     func_args = list(inspect.signature(function).parameters.values())[1:]
     custom_kwargs = []

     args = command.replace('“','"').replace('”','"').replace("‘","\"").replace("’","\"").split() #filtering

     if args and args[0] == name:
         args.pop(0)
         if not args:
             return []

     refined = []
     in_quotes = False
     current_arg = ""

     for arg in args:
         if in_quotes:
             current_arg += " " + arg
             if arg.endswith("\""):
                 in_quotes = False
                 refined.append(current_arg.strip("\""))
                 current_arg = ""
             continue

         if arg.startswith("\""):
             in_quotes = True
             current_arg = arg.strip("\"")
         else:
             refined.append(arg)

     role_mention_idx = 0
     mention_idx = 0
     idx = 0
     for arg_name, arg in zip(func_args, refined):
         if idx >= len(refined):
             break

         if arg_name.annotation == str:
             custom_kwargs.append(str(arg))
         elif arg_name.annotation == int:
           try:
             custom_kwargs.append(int(arg))
           except Exception:
             print(f"Ignoring exception: Provided argument '{arg}' is not an integer.")
         elif arg_name.annotation == Role:
           try:
             custom_kwargs.append(role_mentions[role_mention_idx])
             role_mention_idx += 1
           except Exception:
             print(f"\033[91mIgnoring exception: Provided argument '{arg}' is not a role mention.\033[0m")
         elif arg_name.annotation == User:
             try:
                 custom_kwargs.append(mentions[mention_idx])
                 mention_idx += 1
             except Exception:
                 print(f"\033[91mIgnoring exception: Provided argument '{arg}' is not a mention.\033[0m")
         else:
             custom_kwargs.append(arg)

         idx += 1
     return custom_kwargs


class State:
 def __init__(self,instance):
   self.instance = instance

 async def update_avatar(self,file):
   headers = {
     'Authorization': f'Bot {self.instance.token}',
     'Content-Type': 'application/json',
     'User-Agent': agent
   }
   img = base64.b64encode(open(file,'rb').read()).decode('utf-8')
   data = {"avatar":f"data:image/png;base64,{img}"}
   e = requests.patch(f"{self.instance.base_url}users/@me/",json=data,headers=headers)
   return e.json()

 async def update_status(self,status=None,game=None):
   self.instance.connection.send(json.dumps({"op":3,"d":{"status":status,"game":{"name":game,"type":1}}}))

 async def add_friend(self,user):
   headers = {
     'Authorization': f'Bot {self.instance.token}',
     'Content-Type': 'application/json',
     'User-Agent': agent
   }
   e = requests.put(f"{self.instance.base_url}users/@me/relationships/{user}",headers=headers,json={"type":False})

class Client:
   def __init__(self, prefix,token,status=None,game=None,url=None,events:Events=None):
     if url == None:
       url = "https://hummus.sys42.net/api/v6/"
       cdn = "https://hummus-cdn.sys42.net/"
     else:
       if not url[(len(url)-1)] == "/":
         url = url + "/"
     if status == None:
       status = "online"
     if url:
       e = requests.get(url+"/gateway",headers={"User-Agent":agent})
       if e.json().get('url'):
         websocket = e.json()['url']
     if url != None:
       cdn = url.split(".")
       cdn[0] = cdn[0]+"-cdn"
       ok = cdn[0]
       cdn.pop(0)
       for i in cdn:
         ok = ok + "." + i
       cdn = ok.replace("api/v6/","")
     if events == None:
       asyncio.run(self.LISTEN(Events(self),silent=True))
     else:
       self.events = events
     self.websocket = websocket+"?api=v6&encoding=json"
     self.prefix = prefix
     self.token = token
     self.game = game
     self.base_url = url
     self.cdn = cdn
     self.status = status
     self.fetch = Fetch(self.token,self.base_url,self.cdn,self)
     self.state = State(self)

   async def LISTEN(self,instance:Events,silent:bool=False):
     self.events = instance
     if not silent:
       print("\033[92mRegistered Events class to listen.\033[0m")

   async def RUN(self):
     reconnect = False
     session = ""
     seq = ""
     self.allGuilds:list[AllGuild] = []
     while True:
      try:
       with connect(self.websocket,user_agent_header=ua.random) as websocket:
         self.connection = websocket
         print("\033[92mrestarting...\033[0m")
         websocket.send(json.dumps({'op': 2,'d': {'token':self.token,'presence': {'status': self.status,'game': {"name":self.game,"type":1}}}}))
         if reconnect:
           event = json.loads(websocket.recv())
           print(f"\033[92mResuming session with sequence ID {seq}\033[0m")
           if event['t'] == "READY":
             websocket.send(json.dumps({"token":self.token,"session_id":session,"seq":seq,"op":6}))

         while True:
           event = json.loads(websocket.recv())
           seq = event['s']

           if event['t'] == "READY":
             session = event['d']['session_id']
             print("\033[92mready, recieving login guild data\033[0m")
           if event['op'] == 1:
             websocket.send(json.dumps({"op":1}))
           if event['t'] == "TYPING_START":
             member = event['d'].get('member')
             if member:
               member = Member(self,event['d']['member'],self.cdn,self.token,self.base_url,event['d']['guild_id'])
             else:
               member = event['d']['user_id']
             await self.events.on_typing_start(event['d']['channel_id'],member,event['d']['timestamp'])
           if event['t'] == "GUILD_MEMBER_UPDATE":
             guild_idx = 0
             member_idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['guild_id']:
                 for member in guild.members:
                   if member.id == event['d']['id']:
                     previous = member
                     new_member = Member(self,event['d'],self.cdn,self.token,self.base_url,event['d']['guild_id'])
                     self.allGuilds[guild_idx].members[member_idx] = new_member
                     await self.events.on_guild_member_update(previous,new_member)
                   member_idx += 1
               guild_idx += 1
           if event['t'] == "GUILD_UPDATE":
             guild_idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['id']:
                 updated_guild = Guild(event['d'],self.cdn,self)
                 previous = await guild.toDict()
                 self.allGuilds[guild_idx].guild = updated_guild
                 await self.events.on_guild_update(previous,updated_guild)
               guild_idx += 1
           if event['t'] == "GUILD_EMOJIS_UPDATE":
             guild_idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['guild_id']:
                 before = guild.emojis
                 guild.emojis = []
                 for emoji in event['d']['emojis']:
                   self.allGuilds[guild_idx].append(Emoji(emoji,self))
                 await self.events.on_guild_emojis_update(before,guild.emojis)
               guild_idx += 1
           if event['t'] == "GUILD_ROLE_UPDATE":
             guild_idx = 0
             role_idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['guild_id']:
                 for role in guild.roles:
                   if role.id == event['d']['role']['id']:
                     previous = role
                     new_role = Role(event['d']['role'])
                     self.allGuilds[guild_idx].roles[role_idx] = new_role
                     await self.events.on_guild_role_update(event['d']['guild_id'],previous,new_role)
                   role_idx += 1
               guild_idx += 1
           if event['t'] == "GUILD_ROLE_CREATE":
             guild_idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['guild_id']:
                 new_role = Role(event['d']['role'])
                 self.allGuilds[guild_idx].roles.append(new_role)
                 await self.events.on_guild_role_create(event['d']['guild_id'],new_role)
               guild_idx += 1
           if event['t'] == "GUILD_ROLE_DELETE":
             guild_idx = 0
             role_idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['guild_id']:
                 for role in guild.roles:
                   if role.id == event['d']['role_id']:
                     deleted_role = role
                     self.allGuilds[guild_idx].roles.pop(role_idx)
                     await self.events.on_guild_role_delete(event['d']['guild_id'],deleted_role)
                   role_idx += 1
               guild_idx += 1
           if event['t'] == "CHANNEL_CREATE":
             guild_idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['guild_id']:
                 new_channel = Channel(event['d'],self)
                 self.allGuilds[guild_idx].channels.append(new_channel)
                 await self.events.on_channel_create(new_channel)
               guild_idx += 1
           if event['t'] == "CHANNEL_UPDATE":
             guild_idx = 0
             channel_idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['guild_id']:
                 for channel in guild.channels:
                   if channel.id == event['d']['id']:
                     updated_channel = Channel(event['d'],self)
                     before = guild.channels[channel_idx]
                     self.allGuilds[guild_idx].channels[channel_idx] = updated_channel
                     await self.events.on_channel_update(before,updated_channel)
                   channel_idx += 1
               guild_idx += 1
           if event['t'] == "CHANNEL_DELETE":
             guild_idx = 0
             channel_idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['guild_id']:
                 for channel in guild.channels:
                   if channel.id == event['d']['id']:
                     deleted_channel = guild.channels[channel_idx]
                     self.allGuilds[guild_idx].channels.pop(channel_idx)
                     await self.events.on_channel_delete(deleted_channel)
                   channel_idx += 1
               guild_idx += 1
           if event['t'] == "PRESENCE_UPDATE" and event['d'].get('guild_id'): #how am i going to update a guild presence if theres no guild id?
               guild_idx = 0
               presence_idx = 0
               for guild in self.allGuilds:
                 if guild.id == event['d']['guild_id']:
                   for presence in guild.presences:
                     if presence.user.id == event['d']['user']['id']:
                       new_presence = Presence(self,event['d'],self.cdn,self.token,self.base_url,event['d'].get('guild_id')) #just in case
                       before = guild.presences[presence_idx]
                       self.allGuilds[guild_idx].presences[presence_idx] = new_presence
                       await self.events.on_presence_update(before,new_presence)
                     presence_idx += 1
                 guild_idx += 1
           if event['t'] == "GUILD_CREATE":
             guild = AllGuild(self,event['d'],self.cdn,self.token,self.base_url)
             self.allGuilds.append(guild)
             await self.events.on_guild_create(guild)

           if event['t'] == "GUILD_DELETE":
             idx = 0
             deleted_guild = None
             for guild in self.allGuilds:
               if guild.id == event['d']['id']:
                 deleted_guild = self.allGuilds[idx]
                 self.allGuilds.pop(idx)
               idx += 1
             await self.events.on_guild_delete(deleted_guild)

           if event['t'] == "GUILD_MEMBER_ADD":
             member = Member(self,event['d'],self.cdn,self.token,self.base_url,event['d']['guild_id'])
             idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['guild_id']:
                 self.allGuilds[idx].members.append(member)
               idx += 1
             await self.events.on_guild_member_add(event['d']['guild_id'],member)

           if event['t'] == "GUILD_MEMBER_REMOVE":
             member = User(self,event['d']['user'],self.base_url,self.token,self.cdn)
             idx = 0
             for guild in self.allGuilds:
               if guild.id == event['d']['guild_id']:
                 midx = 0
                 for member in self.allGuilds[idx].members:
                   if member.id == event['d']['user']['id']:
                     self.allGuilds[idx].members.pop(midx)
               idx += 1
             await self.events.on_guild_member_remove(event['d']['guild_id'],member)

           if event['t'] == "MESSAGE_DELETE":
             await self.events.on_message_delete(event['d']['id'],event['d']['channel_id'],event['d']['guild_id'])
             await self.fetch.getMessage(event['d']['channel_id'],event['d']['id'])
           if event['t'] == "MESSAGE_CREATE":
             await self.events.on_message_create(Message(event['d'],self.token,agent,self.base_url,self.cdn,self))
             if event['d']['content'].startswith(self.prefix):
               try:
                 message = Message(event['d'],self.token,agent,self.base_url,self.cdn,self)
                 func = message.content.replace(self.prefix,"")
                 args = func.split(" ")
                 thing = getattr(self, args[0])
                 if callable(thing) and not "__" in args[0] and args[0] != "RUN" and args[0] != "LISTEN":
                   try:
                     command_args = splitArgs(thing,message.content,self.prefix+args[0],self,message.mentions,message.mention_roles)
                     await thing(message,*command_args)
                   except Exception as e:
                     print(f"\033[91mIgnoring exception: {e} \n Traceback: \033[0m")
                     traceback.print_exc()
               except AttributeError as e:
                 if args[0] in str(e):
                   print(f"\033[91mCommand '{args[0]}' not found as a command.\033[0m")
      except Exception as e:
        print(f"\033[91mDisconnected with error {e}, reconnecting...\033[0m")
        traceback.print_exc()
        self.allGuilds = []
        reconnect = True


from .permissions import Permissions
from .member import Avatar, User
from .role import Role

import requests
import fake_useragent
import json

ua = fake_useragent.UserAgent(browsers=['chrome', "firefox", "opera", "safari", "edge", "internet explorer"])

headers = {"Authorization":"","Content-Type":"application/json","User-Agent":ua.random}

class Guild:
  def __init__(self,data,cdn,instance):
    self.instance = instance
    self.id = data['id']
    self.name = data['name']
    self.icon = Avatar(self.id,data['icon'],cdn)
    self.owner_id = data['owner_id']
    self.features = data['features']
    self.region = data['region']
    self.roles = []
    for role in data['roles']:
      self.roles.append(role)
    self.cdn = cdn
    self.emojis = []
    for emoji in data['emojis']:
      self.emojis.append(emoji)
    self.voice_states = []
    for voice_state in data['voice_states']:
      self.voice_states.append(voice_state)
    self.afk_timeout = data['afk_timeout']

class PermOverwrites:
  def __init__(self,data):
    if type(data) == dict:
      self.id = data['id']
      self.type = data['type']
      self.allow = Permissions(data['allow'])
      self.deny = Permissions(data['deny'])
    if type(data) == Role:
      self.id = data.id
      self.type = "role"
      self.allow = Permissions(data.permissions)
      self.deny = Permissions(0)

  def toDict(self):
    self.allow.update()
    self.deny.update()
    return {"id":self.id,"type":self.type,"allow":self.allow.value,"deny":self.deny.value}

class Channel:
  def __init__(self,data,instance):
    self.headers = headers
    self.headers['Authorization'] = f"Bot {instance.token}"
    self.instance = instance
    if data['type'] == 0:
      self.topic = data['topic']
      self.last_message_id = data['last_message_id']
      self.nsfw = bool(data.get('nsfw'))
    if data['type'] == 2:
      self.bitrate = data['bitrate']
      self.user_limit = data.get('user_limit')
    self.type = data['type']
    self.id = data['id']
    self.guild_id = data['guild_id']
    self.name = data['name']
    self.permission_overwrites = []
    for overwrite in data['permission_overwrites']:
      self.permission_overwrites.append(PermOverwrites(overwrite))
    self.position = data['position']

  async def toDict(self):
    if self.type == 0:
      return {"name":self.name,"permission_overwrites":self.permission_overwrites,"position":self.position,"topic":self.topic,"nsfw":self.nsfw,"last_message_id":self.last_message_id,"type":self.type,"id":self.id,"guild_id":self.guild_id}
    elif self.type == 2:
      return {"bitrate":self.bitrate,"name":self.name,"permission_overwrites":self.permission_overwrites,"position":self.position,"user_limit":self.user_limit,"type":self.type,"id":self.id,"guild_id":self.guild_id}

  async def changeName(self,name):
    headers['Authorization'] = f"Bot {self.instance.token}"
    data = json.dumps({'name': name})
    e = requests.patch(url=f"{self.instance.base_url}/channels/{self.id}",headers=headers,data=data)
    if e.json().get('code') == 403:
      raise Exception("Bot does not have 'Manage Channels' permissions")
    return Channel(e.json(),self.instance)

  async def changePerms(self,role:Role,overwrite:PermOverwrites):
    if type(role) != Role:
      raise Exception("Please provide a Role object.")
    if type(overwrite) != PermOverwrites:
      raise Exception("Please provide a Permissions Overwrite (PermOverwrites) object.")
    role.permissions.update()
    overwrite.allow.update()
    overwrite.deny.update()
    data = {"id":role.id,"type":overwrite.type,"allow":overwrite.allow.value,"deny":overwrite.deny.value}
    headers['Authorization'] = f"Bot {self.instance.token}"
    e = requests.put(url=f"{self.instance.base_url}channels/{self.id}/permissions/{role.id}",headers=headers,json=data)
    if e.status_code == 204:
      return
    if e.json().get('code') == 403:
      raise Exception("Bot does not have 'Manage Channels' permissions")

class Emoji:
  def __init__(self,data,instance):
    self.id = data['id']
    self.name = data['name']
    self.guild_id = data['guild_id']
    self.roles = []
    for role in data['roles']:
      self.roles.append(role)
    self.user = User(instance,data['user'],instance.cdn,instance.token,instance.base_url,self.guild_id)
    self.require_colons = data['require_colons']
    self.managed = data['managed']
    self.allNamesString = data['allNamesString']

  async def toDict(self):
    return {"id":self.id,"name":self.name,"roles":self.roles,"user":self.user.toDict(),"require_colons":self.require_colons,"managed":self.managed,"allNamesString":self.allNamesString}


from requests_toolbelt import MultipartEncoder
import tempfile
import filetype
import random
import string
import json
import io

class File:
	def __init__(self,file):
		read = None
		filename = None
		thing = None

		if type(file) == io.BufferedReader:
			file = file.read() #the lines below should rake care of the rest

		if type(file) == bytes:
			with tempfile.NamedTemporaryFile(delete=True) as temp_file:
				temp_file.write(file)
				kind = filetype.guess(temp_file.name)
			filename = f"image.{kind.extension}"
			thing = (filename, file, kind.mime)
			read = file

		if type(file) == str:
			openedfile = open(file,"rb")
			kind = filetype.guess(file)
			read = openedfile.read()
			filename = file.split("/")[-1]
			thing = (file, read, kind.mime)

		if type(file) == io.BytesIO:
			kind = filetype.guess(file.getvalue())
			filename = f"image.{kind.extension}"
			read = file.getvalue()
			thing = (filename,read, kind.mime)

		self.headers = {
		'Content-Type': 'multipart/form-data; boundary=418737004913864675834237162763',
		} #this will be filled with token and useragent before message send
		self.file_json = {"filename":filename,"Content-Type":"image/png"}
		self.fields = {"file":thing,"payload_json":None} #"payload_json" will also be filled later along with token and useragent
		fakedata = {'content': "",'tts':False,'file':self.file_json}
		fakefields = {"file":thing,"payload_json":json.dumps(fakedata)}
		self.data = MultipartEncoder(fields=fakefields,boundary='----WebKitFormBoundary'+''.join(random.sample(string.ascii_letters+string.digits,16)))
		self.headers['Content-Type'] = self.data.content_type


from setuptools import setup, find_packages

"""
python setup.py sdist bdist_wheel
twine upload dist/*
"""

setup(
				name='hummus2016.py',
				version='0.6.0',
				author='LG125YT',
				author_email='lg125yt@gmail.com',
				description='An asynchronous API wrapper for Hummus by Ziad87',
				long_description=open('README.md').read(),
				long_description_content_type='text/markdown',
				url='https://gitlab.com/lg125yt/hummus.py',
				packages=find_packages(),
				install_requires=[
								"discord.py",
								"asyncio",
								"fake_useragent",
								"websockets",
								"requests",
								"requests_toolbelt",
								"filetype",
								"Pillow",
				],
				classifiers=[
								'Development Status :: 3 - Alpha',
								'Intended Audience :: Developers',
								'License :: OSI Approved :: MIT License',
								'Operating System :: OS Independent',
								'Programming Language :: Python',
								'Programming Language :: Python :: 3',
								'Programming Language :: Python :: 3.6',
								'Programming Language :: Python :: 3.7',
								'Programming Language :: Python :: 3.8',
								'Programming Language :: Python :: 3.9',
								'Programming Language :: Python :: 3.10',
								'Programming Language :: Python :: 3.11',
				],
)


from .allguild import *
from .events import *
from .fetch import *
from .funcs import *
from .guild import *
from .main import *
from .member import *
from .message import *
from .permissions import *
from .role import *
from .embed import *
from .file import *

#so the idiots who made py-right extended dont have to be harassed even more because of "could not find [random attrubute] in 'hummus'"

# but like seriously what the hell is "imports is un-sorted/formatted"!?!?!?!?